@@ -25,7 +25,7 @@ try:
 except ImportError:
     # windows platform
     class NoSuchProcess(Exception): pass
-    
+
     def ProcInfo(pid):
         raise NoSuchProcess()
 
@@ -34,7 +34,7 @@ def chown(path, login=None, group=None):
     """Same as `os.chown` function but accepting user login or group name as
     argument. If login or group is omitted, it's left unchanged.
 
-    Note: you must own the file to chown it (or be root). Otherwise OSError is raised. 
+    Note: you must own the file to chown it (or be root). Otherwise OSError is raised.
     """
     if login is None:
         uid = -1
@@ -51,9 +51,8 @@ def chown(path, login=None, group=None):
             gid = int(group)
         except ValueError:
             import grp
-            gid = grp.getgrname(group).gr_gid
+            gid = grp.getgrnam(group).gr_gid
     os.chown(path, uid, gid)
-        
 
 def mv(source, destination, _action=shutil.move):
     """A shell-like mv, supporting wildcards.
@@ -75,7 +74,7 @@ def mv(source, destination, _action=shutil.move):
         except OSError, ex:
             raise OSError('Unable to move %r to %r (%s)' % (
                 source, destination, ex))
-        
+
 def rm(*files):
     """A shell-like rm, supporting wildcards.
     """
@@ -87,13 +86,12 @@ def rm(*files):
                 shutil.rmtree(filename)
             else:
                 os.remove(filename)
-    
+
 def cp(source, destination):
     """A shell-like cp, supporting wildcards.
     """
     mv(source, destination, _action=shutil.copy)
 
-
 def find(directory, exts, exclude=False, blacklist=STD_BLACKLIST):
     """Recursivly find files ending with the given extensions from the directory.
 
@@ -156,7 +154,7 @@ def globfind(directory, pattern, blacklist=STD_BLACKLIST):
     """Recursively finds files matching glob `pattern` under `directory`.
 
     This is an alternative to `logilab.common.shellutils.find`.
-    
+
     :type directory: str
     :param directory:
       directory where the search should start
@@ -198,7 +196,7 @@ class Execute:
     """This is a deadlock safe version of popen2 (no stdin), that returns
     an object with errorlevel, out and err.
     """
-    
+
     def __init__(self, command):
         outfile = tempfile.mktemp()
         errfile = tempfile.mktemp()
@@ -258,9 +256,9 @@ def release_lock(lock_file):
 
 class ProgressBar(object):
     """A simple text progression bar."""
-    
-    def __init__(self, nbops, size=20, stream=sys.stdout):
-        self._fstr = '\r[%%-%ss]' % int(size)
+
+    def __init__(self, nbops, size=20, stream=sys.stdout, title=''):
+        self._fstr = '\r%s[%%-%ss]' % (title, int(size))
         self._stream = stream
         self._total = nbops
         self._size = size
@@ -279,3 +277,19 @@ class ProgressBar(object):
         """Refresh the progression bar display."""
         self._stream.write(self._fstr % ('.' * min(self._progress, self._size)) )
         self._stream.flush()
+
+def confirm(question, default_is_yes=True):
+    """ask for confirmation and return true on positive answer"""
+    if default_is_yes:
+        input_str = '%s [Y/n]: '
+    else:
+        input_str = '%s [y/N]: '
+    answer = raw_input(input_str % (question)).strip().lower()
+    if default_is_yes:
+        if answer in ('n', 'no'):
+            return False
+        return True
+    if answer in ('y', 'yes'):
+        return True
+    return False
+
